@@ -22,7 +22,7 @@
 
     Change History (most recent first):
 
-$Log: mDNSUNP.c,v $
+$Log$
 Revision 1.2.2.3  2005/09/11 19:13:48  rpedde
 mdns fix for freebsd
 
