@@ -1,5 +1,5 @@
 /*
- * $Id: uici.h 964 2006-04-17 04:56:22Z rpedde $
+ * $Id: uici.h 132 2003-12-29 20:41:08Z ron $
  * Sockets Library
  *
  * ** NOTICE **
@@ -37,13 +37,7 @@
 /*   Prototypes for the three public UICI functions                  */  
 /*********************************************************************/
 #define UPORT
-
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-
 typedef unsigned short u_port_t;
 int u_open(u_port_t port);
-int u_accept(int fd, struct in_addr *hostaddr);
+int u_accept(int fd, char *hostn, int hostnsize);
 int u_connect(u_port_t port, char *hostn);
