@@ -1,5 +1,5 @@
 /*
- * $Id: webserver.c 1116 2006-05-28 06:11:37Z rpedde $
+ * $Id: webserver.c 1717 2008-04-19 19:57:51Z rpedde $
  * Webserver library
  *
  * Copyright (C) 2003 Ron Pedde (ron@pedde.com)
@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#  include "config.h"
 #endif
 
 #include <ctype.h>
@@ -29,33 +29,27 @@
 #include <limits.h>
 #include <pthread.h>
 #include <regex.h>
+#include <restart.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <uici.h>
 
-#ifndef WIN32
-# include <sys/param.h>
-# include <sys/types.h>
-# include <sys/socket.h>
-#endif
+#include <sys/param.h>
+#include <sys/types.h>
+#include <sys/socket.h>
 
 #include "err.h"
-#include "restart.h"
 #include "webserver.h"
 
-#ifndef WIN32  /* FIXME: the uici stuff should be moved into os-win32 and os-unix
-                * as os_opensocket, os_accept, etc */
-# include "uici.h"
-#endif
-
 /*
  * Defines
  */
 
 #define MAX_HOSTNAME 256
-#define MAX_LINEBUFFER 2048
+#define MAX_LINEBUFFER 4096
 
 /*
  * Local (private) typedefs
@@ -64,13 +58,14 @@
 typedef struct tag_ws_handler {
     regex_t regex;
     void (*req_handler)(WS_CONNINFO*);
-    int(*auth_handler)(WS_CONNINFO*, char *, char *);
+    int(*auth_handler)(char *, char *);
     int addheaders;
     struct tag_ws_handler *next;
 } WS_HANDLER;
 
 typedef struct tag_ws_connlist {
     WS_CONNINFO *pwsc;
+    char *status;
     struct tag_ws_connlist *next;
 } WS_CONNLIST;
 
@@ -105,13 +100,13 @@ int ws_getpostvars(WS_CONNINFO *pwsc);
 int ws_getgetvars(WS_CONNINFO *pwsc, char *string);
 char *ws_getarg(ARGLIST *root, char *key);
 int ws_testarg(ARGLIST *root, char *key, char *value);
-int ws_findhandler(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc, 
+int ws_findhandler(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc,
                    void(**preq)(WS_CONNINFO*),
-                   int(**pauth)(WS_CONNINFO*, char *, char *),
+                   int(**pauth)(char *, char *),
                    int *addheaders);
-int ws_registerhandler(WSHANDLE ws, char *regex, 
+int ws_registerhandler(WSHANDLE ws, char *regex,
                        void(*handler)(WS_CONNINFO*),
-                       int(*auth)(WS_CONNINFO*, char *, char *),
+                       int(*auth)(char *, char *),
                        int addheaders);
 int ws_decodepassword(char *header, char **username, char **password);
 int ws_testrequestheader(WS_CONNINFO *pwsc, char *header, char *value);
@@ -175,19 +170,6 @@ int ws_unlock_unsafe(void) {
     return retval;
 }
 
-/** 
- * lock the connection list
- */
-void ws_lock_connlist(WS_PRIVATE *pwsp) {
-    if(pthread_mutex_lock(&pwsp->exit_mutex))
-        DPRINTF(E_FATAL,L_WS,"Cannot lock condition mutex\n");
-}
-
-void ws_unlock_connlist(WS_PRIVATE *pwsp) {
-    pthread_mutex_unlock(&pwsp->exit_mutex);
-}
-
-
 /*
  * ws_start
  *
@@ -203,12 +185,10 @@ void ws_unlock_connlist(WS_PRIVATE *pwsp) {
  */
 WSHANDLE ws_start(WSCONFIG *config) {
     int err;
-    int ephemeral = 0;
-
     WS_PRIVATE *pwsp;
 
     DPRINTF(E_SPAM,L_WS,"Entering ws_start\n");
-    
+
     if((pwsp=(WS_PRIVATE*)malloc(sizeof(WS_PRIVATE))) == NULL) {
         DPRINTF(E_SPAM,L_WS,"Malloc error: %s\n",strerror(errno));
         return NULL;
@@ -234,38 +214,15 @@ WSHANDLE ws_start(WSCONFIG *config) {
         return NULL;
     }
 
-    /* this is kind of stupid, but is easier to fix once here
-     * rather than mucking with differences in sockets on windows/unix
-     */
+    DPRINTF(E_INF,L_WS,"Preparing to listen on port %d\n",pwsp->wsconfig.port);
 
-    if(pwsp->wsconfig.port == 0) {
-        pwsp->wsconfig.port = 1024; /* start with default port */
-        ephemeral = 1;
+    if((pwsp->server_fd = u_open(pwsp->wsconfig.port)) == -1) {
+        err=errno;
+        DPRINTF(E_LOG,L_WS,"Could not open port: %s\n",strerror(errno));
+        errno=err;
+        return NULL;
     }
 
-    while(1) {
-        DPRINTF(E_INF,L_WS,"Listening on port %d\n",pwsp->wsconfig.port);
-        pwsp->server_fd = u_open(pwsp->wsconfig.port);
-        if(pwsp->server_fd == -1) {
-            if((!ephemeral) || (errno != EADDRINUSE)) {
-                err = errno;
-                DPRINTF(E_LOG,L_WS,"Listen port: %s\n",strerror(errno));
-                errno = err;
-                return NULL;
-            }
-        } else {
-            break;
-        }
-
-        pwsp->wsconfig.port++;
-        if(!pwsp->wsconfig.port) {
-            DPRINTF(E_LOG,L_WS,"Exhausted ports\n");
-            return NULL;
-        }
-    }
-
-    config->port = pwsp->wsconfig.port;
-
     DPRINTF(E_INF,L_WS,"Starting server thread\n");
     if((err=pthread_create(&pwsp->server_tid,NULL,ws_mainthread,(void*)pwsp))) {
         DPRINTF(E_LOG,L_WS,"Could not spawn thread: %s\n",strerror(err));
@@ -273,7 +230,7 @@ WSHANDLE ws_start(WSCONFIG *config) {
         errno=err;
         return NULL;
     }
-    
+
     /* we're really running */
     pwsp->running=1;
 
@@ -284,7 +241,7 @@ WSHANDLE ws_start(WSCONFIG *config) {
 
 /*
  * ws_remove_dispatch_thread
- * 
+ *
  * remove a dispatch thread from the thread list
  */
 void ws_remove_dispatch_thread(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc) {
@@ -292,7 +249,8 @@ void ws_remove_dispatch_thread(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc) {
 
     DPRINTF(E_SPAM,L_WS,"Entering ws_remove_dispatch_thread\n");
 
-    ws_lock_connlist(pwsp);
+    if(pthread_mutex_lock(&pwsp->exit_mutex))
+        DPRINTF(E_FATAL,L_WS,"Cannot lock condition mutex\n");
 
     pTail=&(pwsp->connlist);
     pHead=pwsp->connlist.next;
@@ -308,14 +266,16 @@ void ws_remove_dispatch_thread(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc) {
                 pwsc->threadno,pwsp->dispatch_threads);
 
         pTail->next = pHead->next;
-        
+
+        if(pHead->status)
+            free(pHead->status);
         free(pHead);
 
         /* signal condition in case something is waiting */
         pthread_cond_signal(&pwsp->exit_cond);
     }
 
-    ws_unlock_connlist(pwsp);
+    pthread_mutex_unlock(&pwsp->exit_mutex);
     DPRINTF(E_SPAM,L_WS,"Exiting ws_remote_dispatch_thread\n");
 }
 
@@ -333,18 +293,20 @@ void ws_add_dispatch_thread(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc) {
     pNew=(WS_CONNLIST*)malloc(sizeof(WS_CONNLIST));
     pNew->next=NULL;
     pNew->pwsc=pwsc;
+    pNew->status=strdup("Initializing");
 
     if(!pNew)
         DPRINTF(E_FATAL,L_WS,"Malloc: %s\n",strerror(errno));
-    
-    ws_lock_connlist(pwsp);
+
+    if(pthread_mutex_lock(&pwsp->exit_mutex))
+        DPRINTF(E_FATAL,L_WS,"Cannot lock condition mutex\n");
 
     /* list is locked... */
     pwsp->dispatch_threads++;
     pNew->next = pwsp->connlist.next;
     pwsp->connlist.next = pNew;
 
-    ws_unlock_connlist(pwsp);
+    pthread_mutex_unlock(&pwsp->exit_mutex);
     DPRINTF(E_SPAM,L_WS,"Exiting ws_add_dispatch_thread\n");
 }
 
@@ -374,13 +336,14 @@ extern int ws_stop(WSHANDLE ws) {
 
     DPRINTF(E_DBG,L_WS,"ws_stop: closing the server fd\n");
     shutdown(pwsp->server_fd,SHUT_RDWR);
-    r_close(pwsp->server_fd);
-    
+    r_close(pwsp->server_fd); /* this should tick off the listener */
+
     /* wait for the server thread to terminate.  SHould be quick! */
     pthread_join(pwsp->server_tid,&result);
 
     /* Give the threads an extra push */
-    ws_lock_connlist(pwsp);
+    if(pthread_mutex_lock(&pwsp->exit_mutex))
+        DPRINTF(E_FATAL,L_WS,"Cannot lock condition mutex\n");
 
     pcl=pwsp->connlist.next;
 
@@ -391,7 +354,7 @@ extern int ws_stop(WSHANDLE ws) {
         if(pcl->pwsc->fd) {
             shutdown(pcl->pwsc->fd,SHUT_RDWR);
             r_close(pcl->pwsc->fd);
-        } 
+        }
         pcl=pcl->next;
     }
 
@@ -401,7 +364,7 @@ extern int ws_stop(WSHANDLE ws) {
         pthread_cond_wait(&pwsp->exit_cond, &pwsp->exit_mutex);
     }
 
-    ws_unlock_connlist(pwsp);
+    pthread_mutex_unlock(&pwsp->exit_mutex);
 
     free(pwsp);
 
@@ -415,7 +378,7 @@ extern int ws_stop(WSHANDLE ws) {
  * Main thread for webserver - this accepts connections
  * and spawns a handler thread for each incoming connection.
  *
- * For a persistant connection, these threads will be 
+ * For a persistant connection, these threads will be
  * long-lived, otherwise, they will terminate as soon as
  * the request has been honored.
  *
@@ -427,8 +390,7 @@ void *ws_mainthread(void *arg) {
     WS_PRIVATE *pwsp = (WS_PRIVATE*)arg;
     WS_CONNINFO *pwsc;
     pthread_t tid;
-    char hostname[MAX_HOSTNAME+1];
-    struct in_addr hostaddr;
+    char hostname[MAX_HOSTNAME];
 
     DPRINTF(E_SPAM,L_WS,"Entering ws_mainthread\n");
 
@@ -443,7 +405,7 @@ void *ws_mainthread(void *arg) {
 
         memset(pwsc,0,sizeof(WS_CONNINFO));
 
-        if((fd=u_accept(pwsp->server_fd,&hostaddr)) == -1) {
+        if((fd=u_accept(pwsp->server_fd,hostname,MAX_HOSTNAME)) == -1) {
             DPRINTF(E_LOG,L_WS,"Dispatcher: accept failed: %s\n",strerror(errno));
             shutdown(pwsp->server_fd,SHUT_RDWR);
             r_close(pwsp->server_fd);
@@ -454,8 +416,6 @@ void *ws_mainthread(void *arg) {
             return NULL;
         }
 
-        strncpy(hostname,inet_ntoa(hostaddr),MAX_HOSTNAME);
-
         pwsc->hostname=strdup(hostname);
         pwsc->fd=fd;
         pwsc->pwsp = pwsp;
@@ -468,7 +428,7 @@ void *ws_mainthread(void *arg) {
         ws_lock_unsafe();
         pwsc->threadno=pwsp->threadno;
         pwsp->threadno++;
-        /* we'll hold the unsafe until the dispatch thread is registered */
+        ws_unlock_unsafe();
 
         /* now, throw off a dispatch thread */
         if((err=pthread_create(&tid,NULL,ws_dispatcher,(void*)pwsc))) {
@@ -479,7 +439,6 @@ void *ws_mainthread(void *arg) {
             ws_add_dispatch_thread(pwsp,pwsc);
             pthread_detach(tid);
         }
-        ws_unlock_unsafe();
     }
 
     DPRINTF(E_SPAM,L_WS,"Exiting ws_mainthred\n");
@@ -498,10 +457,15 @@ void *ws_mainthread(void *arg) {
  * allocated memory has been freed
  */
 void ws_close(WS_CONNINFO *pwsc) {
-    WS_PRIVATE *pwsp = (WS_PRIVATE *)(pwsc->pwsp);
+    WS_PRIVATE *pwsp = (WS_PRIVATE *)pwsc->pwsp;
+
 
     DPRINTF(E_SPAM,L_WS,"Entering ws_close\n");
 
+    /* DWB: update the status so it doesn't fill up with no longer
+       relevant entries  */
+    config_set_status(pwsc, 0, NULL);
+
     DPRINTF(E_DBG,L_WS,"Thread %d: Terminating\n",pwsc->threadno);
     DPRINTF(E_DBG,L_WS,"Thread %d: Freeing request headers\n",pwsc->threadno);
     ws_freearglist(&pwsc->request_headers);
@@ -513,26 +477,16 @@ void ws_close(WS_CONNINFO *pwsc) {
         free(pwsc->uri);
         pwsc->uri=NULL;
     }
-    
+
     if((pwsc->close)||(pwsc->error)) {
         DPRINTF(E_DBG,L_WS,"Thread %d: Closing fd\n",pwsc->threadno);
         shutdown(pwsc->fd,SHUT_RDWR);
         r_close(pwsc->fd);
-        /* this thread is done */
+        free(pwsc->hostname);
 
+        /* this thread is done */
         ws_remove_dispatch_thread(pwsp, pwsc);
 
-        /* Get rid of the local storage */
-        if(pwsc->local_storage) {
-            if(pwsc->storage_callback) {
-                pwsc->storage_callback(pwsc->local_storage);
-                pwsc->local_storage=NULL;
-                pwsc->storage_callback=NULL;
-            }
-        }
-    
-        free(pwsc->hostname);
-        memset(pwsc,0x00,sizeof(WS_CONNINFO));
         free(pwsc);
         DPRINTF(E_SPAM,L_WS,"Exiting ws_close (thread terminating)\n");
         pthread_exit(NULL);
@@ -597,7 +551,12 @@ int ws_getpostvars(WS_CONNINFO *pwsc) {
         return -1;
     }
 
-    length=atoi(content_length);
+    length=strtol(content_length, NULL, 10);
+    if(EINVAL == errno || UINT_MAX -1 <= length) {
+        DPRINTF(E_WARN,L_WS, "Thread %d: Suspicious Content-Length value, ignoring request\n",pwsc->threadno);
+        return -1;
+    }
+
     DPRINTF(E_DBG,L_WS,"Thread %d: Post var length: %d\n",
             pwsc->threadno,length);
 
@@ -622,7 +581,7 @@ int ws_getpostvars(WS_CONNINFO *pwsc) {
     DPRINTF(E_DBG,L_WS,"Thread %d: Read post vars: %s\n",pwsc->threadno,buffer);
 
     pwsc->error=ws_getgetvars(pwsc,buffer);
-    
+
     free(buffer);
 
     DPRINTF(E_SPAM,L_WS,"Exiting ws_getpostvars\n");
@@ -651,7 +610,7 @@ int ws_getheaders(WS_CONNINFO *pwsc) {
             DPRINTF(E_INF,L_WS,"Thread %d: Unexpected close\n",pwsc->threadno);
             return -1;
         }
-        
+
         DPRINTF(E_DBG,L_WS,"Thread %d: Read: %s",pwsc->threadno,buffer);
 
         first=buffer;
@@ -670,7 +629,7 @@ int ws_getheaders(WS_CONNINFO *pwsc) {
             last=first;
             strsep(&last,":");
 
-            if(last==first) {
+            if(!last) {
                 DPRINTF(E_WARN,L_WS,"Thread %d: Invalid header: %s\n",
                         pwsc->threadno,first);
             } else {
@@ -707,7 +666,7 @@ int ws_encoding_hack(WS_CONNINFO *pwsc) {
 
     user_agent=ws_getrequestheader(pwsc, "user-agent");
     if(user_agent) {
-        if(strncasecmp(user_agent,"Roku",4) == 0) 
+        if(strncasecmp(user_agent,"Roku",4) == 0)
             space_as_plus=0;
         if(strncasecmp(user_agent,"iTunes",6) == 0)
             space_as_plus=0;
@@ -720,9 +679,10 @@ int ws_encoding_hack(WS_CONNINFO *pwsc) {
  * ws_getgetvars
  *
  * parse a GET string of variables (or POST)
- * 
+ *
  */
 int ws_getgetvars(WS_CONNINFO *pwsc, char *string) {
+    char *new_string;
     char *first, *last, *middle;
     char *key, *value;
     int done;
@@ -737,19 +697,19 @@ int ws_getgetvars(WS_CONNINFO *pwsc, char *string) {
     done=0;
 
     first=string;
-    
+
     while((!done) && (first)) {
         last=middle=first;
         strsep(&last,"&");
         strsep(&middle,"=");
-        
+
         if(!middle) {
             DPRINTF(E_WARN,L_WS,"Thread %d: Bad arg: %s\n",
                     pwsc->threadno,first);
         } else {
             key=ws_urldecode(first,space_as_plus);
             value=ws_urldecode(middle,space_as_plus);
-            
+
             DPRINTF(E_DBG,L_WS,"Thread %d: Adding arg %s = %s\n",
                     pwsc->threadno,key,value);
             ws_addarg(&pwsc->request_vars,key,"%s",value);
@@ -757,7 +717,7 @@ int ws_getgetvars(WS_CONNINFO *pwsc, char *string) {
             free(key);
             free(value);
         }
-        
+
         if(!last) {
             DPRINTF(E_DBG,L_WS,"Thread %d: Done parsing GET/POST args!\n",
                     pwsc->threadno);
@@ -791,15 +751,11 @@ void *ws_dispatcher(void *arg) {
     time_t now;
     struct tm now_tm;
     void (*req_handler)(WS_CONNINFO*);
-    int(*auth_handler)(WS_CONNINFO*, char *, char *);
+    int(*auth_handler)(char *, char *);
 
     DPRINTF(E_DBG,L_WS,"Thread %d: Entering ws_dispatcher (Connection from %s)\n",
             pwsc->threadno, pwsc->hostname);
 
-    /* quick fence to ensure that we have been registered on the thread list */
-    ws_lock_unsafe();
-    ws_unlock_unsafe();
-        
     while(!connection_done) {
         /* Now, get the request from the other end
          * and decide where to dispatch it
@@ -828,7 +784,7 @@ void *ws_dispatcher(void *arg) {
             DPRINTF(E_SPAM,L_WS,"Error: bad request.  Exiting ws_dispatcher\n");
             return NULL;
         }
-        
+
         if(!strcasecmp(first,"get")) {
             pwsc->request_type = RT_GET;
         } else if(!strcasecmp(first,"post")) {
@@ -846,7 +802,7 @@ void *ws_dispatcher(void *arg) {
         first=last;
         strsep(&last," ");
         pwsc->uri=strdup(first);
-        
+
         /* Get headers */
         if((ws_getheaders(pwsc)) || (!last)) { /* didn't provide a HTTP/1.x */
             /* error already set */
@@ -879,21 +835,21 @@ void *ws_dispatcher(void *arg) {
             ws_close(pwsc);
             return NULL;
         }
-        
+
         /* trim the URI */
         first=pwsc->uri;
         strsep(&first,"?");
-        
+
         if(first) { /* got some GET args */
             DPRINTF(E_DBG,L_WS,"Thread %d: parsing GET args\n",pwsc->threadno);
             ws_getgetvars(pwsc,first);
         }
 
         /* fix the URI by un urldecoding it */
-        
+
         DPRINTF(E_DBG,L_WS,"Thread %d: Original URI: %s\n",
                 pwsc->threadno,pwsc->uri);
-        
+
         first=ws_urldecode(pwsc->uri,ws_encoding_hack(pwsc));
         free(pwsc->uri);
         pwsc->uri=first;
@@ -909,14 +865,20 @@ void *ws_dispatcher(void *arg) {
                 pwsc->uri=first;
             }
         }
-        
-        
+
+
         DPRINTF(E_DBG,L_WS,"Thread %d: Translated URI: %s\n",pwsc->threadno,
                 pwsc->uri);
 
         /* now, parse POST args */
-        if(pwsc->request_type == RT_POST)
-            ws_getpostvars(pwsc);
+        if((pwsc->request_type == RT_POST) && (ws_getpostvars(pwsc) == -1)) {
+            DPRINTF(E_LOG,L_WS,"Couldn't process post vars.  Aborting connection\n");
+            pwsc->error=0;
+            pwsc->close=1; /* force a full close */
+            ws_returnerror(pwsc,500,"Internal server error");
+            ws_close(pwsc);
+            return NULL;
+        }
 
         hdrs=1;
 
@@ -927,7 +889,7 @@ void *ws_dispatcher(void *arg) {
                 pwsc->threadno,now);
         gmtime_r(&now,&now_tm);
         DPRINTF(E_DBG,L_WS,"Thread %d: Setting time header\n",pwsc->threadno);
-        ws_addarg(&pwsc->response_headers,"Date", 
+        ws_addarg(&pwsc->response_headers,"Date",
                   "%s, %d %s %d %02d:%02d:%02d GMT",
                   ws_dow[now_tm.tm_wday],now_tm.tm_mday,
                   ws_moy[now_tm.tm_mon],now_tm.tm_year + 1900,
@@ -936,10 +898,10 @@ void *ws_dispatcher(void *arg) {
         if(hdrs) {
             ws_addarg(&pwsc->response_headers,"Connection",
                       pwsc->close ? "close" : "keep-alive");
-            
+
             ws_addarg(&pwsc->response_headers,"Server",
                       "mt-daapd/" VERSION);
-            
+
             ws_addarg(&pwsc->response_headers,"Content-Type","text/html");
             ws_addarg(&pwsc->response_headers,"Content-Language","en_us");
         }
@@ -954,21 +916,20 @@ void *ws_dispatcher(void *arg) {
                     pwsc->threadno);
 
             can_dispatch=0;
-            /* If an auth handler is registered, but it accepts a 
-             * username and password of NULL, then don't bother 
-             * authing. 
+            /* If an auth handler is registered, but it accepts a
+             * username and password of NULL, then don't bother
+             * authing.
              */
-            if((auth_handler) && (auth_handler(pwsc,NULL,NULL)==0)) {
+            if((auth_handler) && (auth_handler(NULL,NULL)==0)) {
                 /* do the auth thing */
                 auth=ws_getarg(&pwsc->request_headers,"Authorization");
-                if(auth) {
-                    ws_decodepassword(auth,&username,&password);
-                    if(auth_handler(pwsc,username,password))
+                if((auth) && (ws_decodepassword(auth,&username,&password) != -1)) {
+                    if(auth_handler(username,password))
                         can_dispatch=1;
-                    ws_addarg(&pwsc->request_vars,"HTTP_USER",username);
-                    ws_addarg(&pwsc->request_vars,"HTTP_PASSWD",password);
+                    ws_addarg(&pwsc->request_vars,"HTTP_USER","%s",username);
+                    ws_addarg(&pwsc->request_vars,"HTTP_PASSWD","%s",password);
                     free(username); /* this frees password too */
-                } 
+                }
 
                 if(!can_dispatch) { /* auth failed, or need auth */
                     //ws_addarg(&pwsc->response_headers,"Connection","close");
@@ -985,7 +946,7 @@ void *ws_dispatcher(void *arg) {
                 if(req_handler)
                     req_handler(pwsc);
                 else
-                    ws_defaulthandler(pwsp,pwsc);               
+                    ws_defaulthandler(pwsp,pwsc);
             }
         }
 
@@ -1019,14 +980,6 @@ int ws_writefd(WS_CONNINFO *pwsc, char *fmt, ...) {
     return r_write(pwsc->fd,buffer,strlen(buffer));
 }
 
-/**
-* write a blob od data to the connfd
-*/
-int ws_writebinary(WS_CONNINFO *pwsc, char *data, int len) {
-    return r_write(pwsc->fd,data,len);
-}
-
-
 
 /*
  * ws_returnerror
@@ -1043,21 +996,15 @@ int ws_returnerror(WS_CONNINFO *pwsc,int error, char *description) {
     DPRINTF(E_WARN,L_WS,"Thread %d: Entering ws_returnerror (%d: %s)\n",
             pwsc->threadno,error,description);
     ws_writefd(pwsc,"HTTP/1.1 %d %s\r\n",error,description);
-    
+
     /* we'll force a close here unless the user agent is
        iTunes, which seems to get pissy about it */
     useragent = ws_getarg(&pwsc->request_headers,"User-Agent");
-    if((useragent) && (strncmp(useragent,"iTunes",6) == 0) && (error == 401)) {
-        ws_addarg(&pwsc->response_headers,"Connection","keep-alive");
-        ws_addarg(&pwsc->response_headers,"Content-Length","2");
-        ws_emitheaders(pwsc);
-        ws_writefd(pwsc,"\r\n");
-        return 0;
+    if(useragent && (strncmp(useragent,"iTunes",6))) {
+        pwsc->close=1;
+        ws_addarg(&pwsc->response_headers,"Connection","close");
     }
 
-    pwsc->close=1;
-    ws_addarg(&pwsc->response_headers,"Connection","close");
-
     ws_emitheaders(pwsc);
 
     ws_writefd(pwsc,"<HTML>\r\n<TITLE>");
@@ -1065,7 +1012,7 @@ int ws_returnerror(WS_CONNINFO *pwsc,int error, char *description) {
     ws_writefd(pwsc,"\r\n<H1>%s</H1>\r\n",description);
     ws_writefd(pwsc,"Error %d\r\n<hr>\r\n",error);
     ws_writefd(pwsc,"<i>mt-daapd: %s\r\n<br>",VERSION);
-    if(errno) 
+    if(errno)
         ws_writefd(pwsc,"Error: %s\r\n",strerror(errno));
 
     ws_writefd(pwsc,"</i></BODY>\r\n</HTML>\r\n");
@@ -1081,14 +1028,14 @@ int ws_returnerror(WS_CONNINFO *pwsc,int error, char *description) {
  * and serves it up
  */
 void ws_defaulthandler(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc) {
-    char path[PATH_MAX];
-    char resolved_path[PATH_MAX];
+    char path[MAXPATHLEN];
+    char resolved_path[MAXPATHLEN];
     int file_fd;
     off_t len;
-    
+
     DPRINTF(E_SPAM,L_WS,"Entering ws_defaulthandler\n");
 
-    snprintf(path,PATH_MAX,"%s/%s",pwsp->wsconfig.web_root,pwsc->uri);
+    snprintf(path,MAXPATHLEN,"%s/%s",pwsp->wsconfig.web_root,pwsc->uri);
     if(!realpath(path,resolved_path)) {
         pwsc->error=errno;
         DPRINTF(E_WARN,L_WS,"Exiting ws_defaulthandler: Cannot resolve %s\n",path);
@@ -1121,7 +1068,7 @@ void ws_defaulthandler(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc) {
         ws_close(pwsc);
         return;
     }
-    
+
     /* set the Content-Length response header */
     len=lseek(file_fd,0,SEEK_END);
 
@@ -1172,20 +1119,14 @@ int ws_testrequestheader(WS_CONNINFO *pwsc, char *header, char *value) {
  */
 int ws_testarg(ARGLIST *root, char *key, char *value) {
     char *retval;
-    int result;
-
 
     DPRINTF(E_DBG,L_WS,"Checking to see if %s matches %s\n",key,value);
 
     retval=ws_getarg(root,key);
-    if(!retval) {
-        DPRINTF(E_DBG,L_WS,"Nope!\n");
+    if(!retval)
         return 0;
-    }
 
-    result=!strcasecmp(value,retval);
-    DPRINTF(E_DBG,L_WS,"And it %s\n",result ? "DOES!" : "does NOT");
-    return result;
+    return !strcasecmp(value,retval);
 }
 
 /*
@@ -1256,7 +1197,7 @@ int ws_addarg(ARGLIST *root, char *key, char *fmt, ...) {
 
     current=root->next;
     while(current) {
-        if(!strcasecmp(current->key,key)) {
+        if(!strcmp(current->key,key)) {
             /* got a match! */
             DPRINTF(E_DBG,L_WS,"Updating %s from %s to %s\n",
                     key,current->value,value);
@@ -1301,6 +1242,10 @@ char *ws_urldecode(char *string, int space_as_plus) {
 
     while(*src) {
         switch(*src) {
+            /* DWB - space gets converted to %20, not +, this definitely breaks compatibility with iTunes */
+            /* But the browsers encode space as plus, so when using the web interface,
+             * anything with a plus is broken.  This will end up having to be sniffed
+             * by remote agent */
         case '+':
             if(space_as_plus) {
                 *dst++=' ';
@@ -1351,9 +1296,9 @@ char *ws_urldecode(char *string, int space_as_plus) {
  * regerror to display a more interesting error message,
  * if appropriate.
  */
-int ws_registerhandler(WSHANDLE ws, char *regex, 
+int ws_registerhandler(WSHANDLE ws, char *regex,
                        void(*handler)(WS_CONNINFO*),
-                       int(*auth)(WS_CONNINFO *, char *, char *),
+                       int(*auth)(char *, char *),
                        int addheaders) {
     WS_HANDLER *phandler;
     WS_PRIVATE *pwsp = (WS_PRIVATE *)ws;
@@ -1388,9 +1333,9 @@ int ws_registerhandler(WSHANDLE ws, char *regex,
  * If a handler is found, it returns 0, otherwise, returns
  * -1
  */
-int ws_findhandler(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc, 
+int ws_findhandler(WS_PRIVATE *pwsp, WS_CONNINFO *pwsc,
                    void(**preq)(WS_CONNINFO*),
-                   int(**pauth)(WS_CONNINFO *, char *, char *),
+                   int(**pauth)(char *, char *),
                    int *addheaders) {
     WS_HANDLER *phandler=pwsp->handlers.next;
 
@@ -1433,6 +1378,7 @@ int ws_decodepassword(char *header, char **username, char **password) {
     int pads=0;
     unsigned char *decodebuffer;
     unsigned char *pin, *pout;
+    char *type,*base64;
     int lookup;
 
     *username=NULL;
@@ -1461,21 +1407,33 @@ int ws_decodepassword(char *header, char **username, char **password) {
         return -1;
 
     /* xlat table is initialized */
-    while(*header != ' ')
+
+    // Trim leading spaces
+    while((*header) && (*header != ' '))
         header++;
 
-    header++;
+     // Should be in the form "Basic <base-64 enc username/pw>"
+    type=header;
+    base64 = strchr(header,' ');
+    if(!base64) {
+        // invalid auth header
+        DPRINTF(E_DBG,L_WS,"Bad authentication header: %s\n",header);
+        return -1;
+    }
 
-    decodebuffer=(unsigned char *)malloc(strlen(header));
+    *base64 = '\0';
+    base64++;
+
+    decodebuffer=(unsigned char *)malloc(strlen(base64));
     if(!decodebuffer)
         return -1;
 
-    DPRINTF(E_DBG,L_WS,"Preparing to decode %s\n",header);
+    DPRINTF(E_DBG,L_WS,"Preparing to decode %s\n",base64);
 
-    memset(decodebuffer,0,strlen(header));
+    memset(decodebuffer,0,strlen(base64));
     len=0;
     pout=decodebuffer;
-    pin=(unsigned char *)header;
+    pin=base64;
 
     /* this is more than a little sloppy */
     while(pin[rack]) {
@@ -1522,7 +1480,7 @@ int ws_decodepassword(char *header, char **username, char **password) {
     /* we now have the decoded string */
     DPRINTF(E_DBG,L_WS,"Decoded %s\n",decodebuffer);
 
-    *username = (char*)decodebuffer;
+    *username = decodebuffer;
     *password = *username;
 
     strsep(password,":");
@@ -1559,104 +1517,3 @@ char *ws_getvar(WS_CONNINFO *pwsc, char *var) {
 char *ws_getrequestheader(WS_CONNINFO *pwsc, char *header) {
     return ws_getarg(&pwsc->request_headers,header);
 }
-
-/**
- * get the local storage pointer.
- *
- * @param pwsc connection to get local storage for
- */
-void *ws_get_local_storage(WS_CONNINFO *pwsc) {
-    return pwsc->local_storage;
-}
-
-
-/**
- * lock the local storage pointer.  This is sort of wrong, as
- * the all operations manipulating local storage are locked,
- * not just the one you are working on.
- * 
- * @param pwsc connection you are working with
- */
-void ws_lock_local_storage(WS_CONNINFO *pwsc) {
-    WS_PRIVATE *pwsp;
-    
-    pwsp = (WS_PRIVATE *)pwsc->pwsp;
-    ws_lock_connlist(pwsp);
-}
-
-void ws_unlock_local_storage(WS_CONNINFO *pwsc) {
-    WS_PRIVATE *pwsp;
-    
-    pwsp = (WS_PRIVATE *)pwsc->pwsp;
-    ws_unlock_connlist(pwsp);
-}
-
-/**
- * set the local storage pointer (and callback)
- */
-void ws_set_local_storage(WS_CONNINFO *pwsc, void *ptr, void (*callback)(void *)) {
-    if(!pwsc)
-        return;
-
-    if(pwsc->local_storage) {
-        DPRINTF(E_FATAL,L_WS,"ls already allocated");
-    }
-    
-    pwsc->storage_callback = callback;
-    pwsc->local_storage = ptr;
-}
-
-/**
- * walk through the connection list and enumerate all the items
- */
-WS_CONNINFO *ws_thread_enum_first(WSHANDLE wsh, WSTHREADENUM *vpp) {
-    WS_PRIVATE *pwsp = (WS_PRIVATE *)wsh;
-    WS_CONNINFO *pwsc = NULL;
-    WS_CONNLIST *pconlist;
-    
-    ws_lock_connlist(pwsp);
-    
-    pconlist = pwsp->connlist.next;
-    *vpp = (WSTHREADENUM)pconlist;
-    if(pconlist) {
-        pwsc = pconlist->pwsc;
-    } else {
-        ws_unlock_connlist(pwsp);
-    }
-    
-    return pwsc;
-}
-
-WS_CONNINFO *ws_thread_enum_next(WSHANDLE wsh, WSTHREADENUM *vpp) {
-    WS_PRIVATE *pwsp = (WS_PRIVATE *)wsh;
-    WS_CONNINFO *pwsc = NULL;
-    WS_CONNLIST *pconlist;
-    
-    pconlist = (WS_CONNLIST*)*vpp;
-    if((!pconlist) || (!pconlist->next)) {
-        ws_unlock_connlist(pwsp);
-    } else {
-        pconlist=pconlist->next;
-        *vpp = (WSTHREADENUM)pconlist;
-        pwsc = pconlist->pwsc;
-    }
-
-    return pwsc;
-}
-
-void *ws_enum_var(WS_CONNINFO *pwsc, char **key, char **value, void *last) {
-    ARGLIST *plist = (ARGLIST *)last;
-
-    if(!plist) {
-        plist = pwsc->request_vars.next;
-    } else {
-        plist = plist->next;
-    }
-
-    if(plist) {
-        if(key) *key = plist->key;
-        if(value) *value = plist->value;
-    }
-
-    return plist;
-}
